@@ -1,5 +1,5 @@
 /*
- * Copyright 2007      by Sascha Hlusiak. <saschahlusiak@freedesktop.org>
+ * Copyright 2007-2008 by Sascha Hlusiak. <saschahlusiak@freedesktop.org>     
  * Copyright 1995-1999 by Frederic Lepied, France. <Lepied@XFree86.org>       
  *                                                                            
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -27,26 +27,37 @@
 #endif
 
 #include <stdio.h>
-#include <xf86Version.h>
+#include <xorgVersion.h>
 
 #include <misc.h>
 #include <xf86.h>
 #include <xf86Xinput.h>
 #include <exevents.h>		/* Needed for InitValuator/Proximity stuff */
+#include <xf86Opt.h>
 
 #include <math.h>
 #include <xf86Module.h>
 
 
 #include "jstk.h"
-#include "jstk_hw.h"
 #include "jstk_axis.h"
 #include "jstk_key.h"
 #include "jstk_options.h"
+#include "jstk_properties.h"
+
+#ifdef LINUX_BACKEND
+    #include "backend_joystick.h"
+#endif
+#ifdef BSD_BACKEND
+    #include "backend_bsd.h"
+#endif
+#ifdef EVDEV_BACKEND
+    #include "backend_evdev.h"
+#endif
 
 
 #if DEBUG
-    int debug_level = 0;
+    char debug_level = 0;
 #endif
 
 
@@ -83,6 +94,44 @@ jstkConvertProc(LocalDevicePtr	local,
 }
 
 
+/*
+ ***************************************************************************
+ *
+ * jstkOpenDevice --
+ *
+ * Called to open the device specified in priv
+ * The compiled backends are tried one by one and return the first matching
+ *
+ * Returns the filedescriptor or -1 in case of error
+ *
+ ***************************************************************************
+ */
+static int
+jstkOpenDevice(JoystickDevPtr priv, BOOL probe)
+{
+    int fd;
+    fd = -1;
+
+    if (probe == FALSE && priv->open_proc)
+        return priv->open_proc(priv, probe);
+
+#ifdef EVDEV_BACKEND
+    if (fd == -1)
+        fd = jstkOpenDevice_evdev(priv, probe);
+#endif
+#ifdef LINUX_BACKEND
+    if (fd == -1)
+        fd = jstkOpenDevice_joystick(priv, probe);
+#endif
+#ifdef BSD_BACKEND
+    if (fd == -1)
+        fd = jstkOpenDevice_bsd(priv, probe);
+#endif
+
+    return fd;
+}
+
+
 /*
  ***************************************************************************
  *
@@ -104,7 +153,8 @@ jstkReadProc(LocalDevicePtr local)
     JoystickDevPtr priv = local->private;
 
     do {
-        if ((r=jstkReadData(priv, &event, &number))==0) {
+        if ((priv->read_proc == NULL) || 
+            ((r=priv->read_proc(priv, &event, &number))==0)) {
             xf86Msg(X_WARNING, "JOYSTICK: Read failed. Deactivating device.\n");
 
             if (local->fd >= 0)
@@ -119,43 +169,43 @@ jstkReadProc(LocalDevicePtr local)
                 priv->button[number].mapping));
 
             switch (priv->button[number].mapping) {
-            case MAPPING_BUTTON:
+            case JSTK_MAPPING_BUTTON:
                 if (priv->mouse_enabled == TRUE) {
-                    xf86PostButtonEvent(local->dev, 0, 
+                    xf86PostButtonEvent(local->dev, 0,
                         priv->button[number].buttonnumber,
                         priv->button[number].pressed, 0, 0);
                 }
                 break;
 
-            case MAPPING_X:
-            case MAPPING_Y:
-            case MAPPING_ZX:
-            case MAPPING_ZY:
+            case JSTK_MAPPING_X:
+            case JSTK_MAPPING_Y:
+            case JSTK_MAPPING_ZX:
+            case JSTK_MAPPING_ZY:
                 if (priv->button[number].pressed == 0) 
                     priv->button[number].currentspeed = 1.0;
                 else if (priv->mouse_enabled == TRUE)
                     jstkStartButtonAxisTimer(local, number);
                 break;
 
-            case MAPPING_KEY:
+            case JSTK_MAPPING_KEY:
                 if (priv->keys_enabled == TRUE)
-                    jstkGenerateKeys(local->dev, 
+                    jstkGenerateKeys(priv->keyboard_device, 
                                      priv->button[number].keys, 
                                      priv->button[number].pressed);
                 break;
 
-            case MAPPING_SPEED_MULTIPLY:
+            case JSTK_MAPPING_SPEED_MULTIPLY:
                 priv->amplify = 1.0;
                 /* Calculate new amplify value by multiplying them all */
                 for (i=0; i<MAXAXES; i++) {
                     if ((priv->button[i].pressed) && 
-                        (priv->button[i].mapping == MAPPING_SPEED_MULTIPLY))
+                        (priv->button[i].mapping == JSTK_MAPPING_SPEED_MULTIPLY))
                         priv->amplify *= priv->button[i].amplify;
                 }
                 DBG(2, ErrorF("Amplify is now %.3f\n", priv->amplify));
                 break;
 
-            case MAPPING_DISABLE:
+            case JSTK_MAPPING_DISABLE:
                 if (priv->button[number].pressed == 1) {
                     if ((priv->mouse_enabled == TRUE) || 
                         (priv->keys_enabled == TRUE))
@@ -170,7 +220,7 @@ jstkReadProc(LocalDevicePtr local)
                     }
                 }
                 break;
-            case MAPPING_DISABLE_MOUSE:
+            case JSTK_MAPPING_DISABLE_MOUSE:
                 if (priv->button[number].pressed == 1) {
                     if (priv->mouse_enabled == TRUE) 
                         priv->mouse_enabled = FALSE;
@@ -179,7 +229,7 @@ jstkReadProc(LocalDevicePtr local)
                         priv->mouse_enabled ? "enabled" : "disabled"));
                 }
                 break;
-            case MAPPING_DISABLE_KEYS:
+            case JSTK_MAPPING_DISABLE_KEYS:
                 if (priv->button[number].pressed == 1) {
                     if (priv->keys_enabled == TRUE) 
                         priv->keys_enabled = FALSE;
@@ -196,7 +246,7 @@ jstkReadProc(LocalDevicePtr local)
 
         /* An axis was moved */
         if ((event == EVENT_AXIS) && 
-            (priv->axis[number].type != TYPE_NONE))
+            (priv->axis[number].type != JSTK_TYPE_NONE))
         {
             DBG(5, ErrorF("Axis %d moved to %d. Type: %d, Mapping: %d\n", 
                           number,
@@ -209,58 +259,56 @@ jstkReadProc(LocalDevicePtr local)
                                     1, priv->axis[number].value);
 
             switch (priv->axis[number].mapping) {
-            case MAPPING_X:
-            case MAPPING_Y:
-            case MAPPING_ZX:
-            case MAPPING_ZY:
+            case JSTK_MAPPING_X:
+            case JSTK_MAPPING_Y:
+            case JSTK_MAPPING_ZX:
+            case JSTK_MAPPING_ZY:
                 switch (priv->axis[number].type) {
-                case TYPE_BYVALUE:
-                case TYPE_ACCELERATED:
+                case JSTK_TYPE_BYVALUE:
+                case JSTK_TYPE_ACCELERATED:
                     if (priv->axis[number].value == 0)
                         priv->axis[number].currentspeed = 1.0;
                     if (priv->mouse_enabled == TRUE)
                         jstkStartAxisTimer(local, number);
                     break;
 
-                case TYPE_ABSOLUTE:
+                case JSTK_TYPE_ABSOLUTE:
                     if (priv->mouse_enabled == TRUE)
                         jstkHandleAbsoluteAxis(local, number);
                     break;
                 default:
                     break;
                 } /* switch (priv->axis[number].type) */
-                break; /* case MAPPING_ZY */
+                break; /* case JSTK_MAPPING_ZY */
 
-            case MAPPING_KEY: if (priv->keys_enabled == TRUE) {
-                if (priv->axis[number].type == TYPE_ACCELERATED) {
-                    if ((priv->axis[number].value > 0) != 
-                        (priv->axis[number].oldvalue > 0))
-                        jstkGenerateKeys(local->dev, 
-                                         priv->axis[number].keys_high,
-                                         (priv->axis[number].value > 0) ? 1:0);
-
-                    if ((priv->axis[number].value < 0) != 
-                        (priv->axis[number].oldvalue < 0))
-                        jstkGenerateKeys(local->dev,
-                                         priv->axis[number].keys_low,
-                                         (priv->axis[number].value < 0) ? 1:0);
-                } else if (priv->axis[number].type == TYPE_BYVALUE) {
-                    if (priv->keys_enabled == TRUE)
-                        jstkStartAxisTimer(local, number);
+            case JSTK_MAPPING_KEY: if (priv->keys_enabled == TRUE) {
+                if (priv->axis[number].type == JSTK_TYPE_ACCELERATED) {
+                    jstkHandlePWMAxis(local, number);
+                } else if (priv->axis[number].type == JSTK_TYPE_BYVALUE) {
+                    jstkStartAxisTimer(local, number);
                 }
                 break;
             }
 
-            case MAPPING_NONE:
+            case JSTK_MAPPING_NONE:
             default:
                 break;
-            } /* switch (priv->axis{number].mapping) */
+            } /* switch (priv->axis[number].mapping) */
         } /* if (event == EVENT_AXIS) */
     } while (r == 2);
 }
 
 
 
+
+
+static void
+jstkPtrCtrlProc(DeviceIntPtr device, PtrCtrl *ctrl)
+{
+    /* Nothing to do, dix handles all settings */
+}
+
+
 /*
  ***************************************************************************
  *
@@ -282,29 +330,40 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
     switch (what) {
     case DEVICE_INIT: {
         int m;
+        CARD8 buttonmap[BUTTONMAP_SIZE+1];
         DBG(1, ErrorF("jstkDeviceControlProc what=INIT\n"));
-        /* We want the first 7 button numbers fixed */
-        if (priv->buttonmap.size != 0) {
-            if (InitButtonClassDeviceStruct(pJstk, priv->buttonmap.size, 
-                priv->buttonmap.map) == FALSE) {
-                ErrorF("unable to allocate Button class device\n");
-                return !Success;
-            }
-            if (InitFocusClassDeviceStruct(pJstk) == FALSE) {
-                ErrorF("unable to init Focus class device\n");
-                return !Success;
-            }
+        /* Probe device and return if error */
+        if (jstkOpenDevice(priv, TRUE) == -1) {
+            return !Success;
+        } else {
+            /* Success. The OpenDevice call already did some initialization
+               like priv->num_buttons, priv->num_axes */
+            priv->close_proc(priv);
         }
-        jstkInitKeys(pJstk, priv);
+
+        for (m=0; m<=BUTTONMAP_SIZE; m++)
+            buttonmap[m] = m;
+        if (InitButtonClassDeviceStruct(pJstk, BUTTONMAP_SIZE, 
+            buttonmap) == FALSE) {
+            ErrorF("unable to allocate Button class device\n");
+            return !Success;
+        }
+        if (!InitPtrFeedbackClassDeviceStruct(pJstk, jstkPtrCtrlProc))
+            return !Success;
 
         m = 2;
         for (i=0; i<MAXAXES; i++) 
-            if (priv->axis[i].type != TYPE_NONE)
-                priv->axis[i].valuator = m++;
+            if (priv->axis[i].valuator != -1)
+        {
+            DBG(3, ErrorF("Axis %d will be valuator %d\n", i, m));
+            priv->axis[i].valuator = m++;
+        }
 
         if (InitValuatorClassDeviceStruct(pJstk, 
                                           m,
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 3
                                           xf86GetMotionEvents, 
+#endif
                                           local->history_size,
                                           Relative) == FALSE) {
             ErrorF("unable to allocate Valuator class device\n"); 
@@ -325,7 +384,7 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
                                    0, /* min_res */
                                    1); /* max_res */
             for (i=0; i<MAXAXES; i++) 
-                if (priv->axis[i].type != TYPE_NONE)
+                if (priv->axis[i].valuator != -1)
             {
                 InitValuatorAxisStruct(pJstk,
                                        priv->axis[i].valuator,
@@ -338,6 +397,22 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
             /* allocate the motion history buffer if needed */
             xf86MotionHistoryAllocate(local);
         }
+
+
+        if (priv->keyboard_device != NULL)
+        {
+            DBG(2, ErrorF("Activating keyboard device\n"));
+            xf86ActivateDevice(priv->keyboard_device);
+            priv->keyboard_device->dev->inited = 
+                (priv->keyboard_device->device_control(priv->keyboard_device->dev, DEVICE_INIT) == Success);
+            xf86EnableDevice(priv->keyboard_device->dev);
+            DBG(2, ErrorF("Keyboard device activated\n"));
+        }
+
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 3
+	jstkInitProperties(pJstk, priv);
+#endif
+
         break;
     }
 
@@ -345,7 +420,7 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
         DBG(1, ErrorF("jstkDeviceControlProc  what=ON name=%s\n", 
                       priv->device));
 
-        if (jstkOpenDevice(priv) != -1) {
+        if (jstkOpenDevice(priv, FALSE) != -1) {
             pJstk->public.on = TRUE;
             local->fd = priv->fd;
             AddEnabledDevice(local->fd);
@@ -363,11 +438,18 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
             priv->timerrunning = FALSE;
             TimerCancel(priv->timer);
         }
+        for (i = 0; i < MAXAXES; i++)
+            if (priv->axis[i].timerrunning)
+        {
+            priv->axis[i].timerrunning = FALSE;
+            TimerCancel(priv->axis[i].timer);
+        }
 
         if (local->fd >= 0)
             RemoveEnabledDevice(local->fd);
         local->fd = -1;
-        jstkCloseDevice(priv);
+        if (priv->close_proc)
+            priv->close_proc(priv);
         pJstk->public.on = FALSE;
         break;
 
@@ -381,6 +463,17 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
 
 
 
+
+_X_EXPORT InputDriverRec JSTK_KEYBOARD = {
+    1,
+    "joystick_keyboard",
+    NULL,
+    jstkKeyboardPreInit,
+    jstkKeyboardUnInit,
+    NULL,
+    0
+};
+
 /*
  ***************************************************************************
  *
@@ -409,7 +502,6 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
 
     local->name   = dev->identifier;
     local->flags  = XI86_POINTER_CAPABLE;
-    local->flags |= XI86_KEYBOARD_CAPABLE;
     local->flags |= XI86_SEND_DRAG_EVENTS;
     local->device_control = jstkDeviceControlProc;
     local->read_input = jstkReadProc;
@@ -420,12 +512,15 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     local->fd = -1;
     local->dev = NULL;
     local->private = priv;
-    local->type_name = XI_MOUSE;
+    local->type_name = XI_JOYSTICK;
     local->history_size = 0;
-    local->always_core_feedback = 0;
+    local->always_core_feedback = NULL;
     local->conf_idev = dev;
 
     priv->fd = -1;
+    priv->open_proc = NULL;
+    priv->read_proc = NULL;
+    priv->close_proc = NULL;
     priv->device = NULL;
     priv->devicedata = NULL;
     priv->timer = NULL;
@@ -433,74 +528,75 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     priv->mouse_enabled = TRUE;
     priv->keys_enabled = TRUE;
     priv->amplify = 1.0f;
-    priv->buttonmap.size = 0;
+    priv->keyboard_device = NULL;
     priv->keymap.size = 1;
-    priv->keymap.map[0] = NoSymbol;
+    memset(priv->keymap.map, NoSymbol, sizeof(priv->keymap.map));
     priv->repeat_delay = 0;
     priv->repeat_interval = 0;
+    priv->num_axes    = MAXAXES;
+    priv->num_buttons = MAXBUTTONS;
 
     /* Initialize default mappings */
     for (i=0; i<MAXAXES; i++) {
         priv->axis[i].value        = 0;
         priv->axis[i].oldvalue     = 0;
-        priv->axis[i].deadzone     = 1000;
-        priv->axis[i].type         = TYPE_NONE;
-        priv->axis[i].mapping      = MAPPING_NONE;
+        priv->axis[i].deadzone     = 5000;
+        priv->axis[i].type         = JSTK_TYPE_NONE;
+        priv->axis[i].mapping      = JSTK_MAPPING_NONE;
         priv->axis[i].currentspeed = 0.0f;
         priv->axis[i].amplify      = 1.0f;
         priv->axis[i].valuator     = -1;
         priv->axis[i].subpixel     = 0.0f;
+        priv->axis[i].timer        = NULL;
+        priv->axis[i].timerrunning = FALSE;
+        priv->axis[i].key_isdown   = 0;
         for (j=0; j<MAXKEYSPERBUTTON; j++)
             priv->axis[i].keys_low[j] = priv->axis[i].keys_high[j] = 0;
     }
     for (i=0; i<MAXBUTTONS; i++) {
         priv->button[i].pressed      = 0;
         priv->button[i].buttonnumber = 0;
-        priv->button[i].mapping      = MAPPING_NONE;
+        priv->button[i].mapping      = JSTK_MAPPING_NONE;
         priv->button[i].currentspeed = 1.0f;
         priv->button[i].subpixel     = 0.0f;
+        priv->button[i].amplify      = 1.0;
         for (j=0; j<MAXKEYSPERBUTTON; j++)
             priv->button[i].keys[j] = 0;
     }
 
     /* First three joystick buttons generate mouse clicks */
-    priv->button[0].mapping      = MAPPING_BUTTON;
+    priv->button[0].mapping      = JSTK_MAPPING_BUTTON;
     priv->button[0].buttonnumber = 1;
-    priv->button[1].mapping      = MAPPING_BUTTON;
+    priv->button[1].mapping      = JSTK_MAPPING_BUTTON;
     priv->button[1].buttonnumber = 2;
-    priv->button[2].mapping      = MAPPING_BUTTON;
+    priv->button[2].mapping      = JSTK_MAPPING_BUTTON;
     priv->button[2].buttonnumber = 3;
 
     /* First two axes are a stick for moving */
-    priv->axis[0].type      = TYPE_BYVALUE;
-    priv->axis[0].mapping   = MAPPING_X;
-    priv->axis[1].type      = TYPE_BYVALUE;
-    priv->axis[1].mapping   = MAPPING_Y;
+    priv->axis[0].type      = JSTK_TYPE_BYVALUE;
+    priv->axis[0].mapping   = JSTK_MAPPING_X;
+    priv->axis[1].type      = JSTK_TYPE_BYVALUE;
+    priv->axis[1].mapping   = JSTK_MAPPING_Y;
 
     /* Next two axes are a stick for scrolling */
-    priv->axis[2].type      = TYPE_BYVALUE;
-    priv->axis[2].mapping   = MAPPING_ZX;
-    priv->axis[3].type      = TYPE_BYVALUE;
-    priv->axis[3].mapping   = MAPPING_ZY;
+    priv->axis[2].type      = JSTK_TYPE_BYVALUE;
+    priv->axis[2].mapping   = JSTK_MAPPING_ZX;
+    priv->axis[3].type      = JSTK_TYPE_BYVALUE;
+    priv->axis[3].mapping   = JSTK_MAPPING_ZY;
 
     /* Next two axes are a pad for moving */
-    priv->axis[4].type      = TYPE_ACCELERATED;
-    priv->axis[4].mapping   = MAPPING_X;
-    priv->axis[5].type      = TYPE_ACCELERATED;
-    priv->axis[5].mapping   = MAPPING_Y;
-
-    priv->buttonmap.scrollbutton[0] = jstkGetButtonNumberInMap(priv, 4);
-    priv->buttonmap.scrollbutton[1] = jstkGetButtonNumberInMap(priv, 5);
-    priv->buttonmap.scrollbutton[2] = jstkGetButtonNumberInMap(priv, 6);
-    priv->buttonmap.scrollbutton[3] = jstkGetButtonNumberInMap(priv, 7);
-    priv->buttonmap.map[0] = 0;
-
+    priv->axis[4].type      = JSTK_TYPE_ACCELERATED;
+    priv->axis[4].mapping   = JSTK_MAPPING_X;
+    priv->axis[5].type      = JSTK_TYPE_ACCELERATED;
+    priv->axis[5].mapping   = JSTK_MAPPING_Y;
 
     xf86CollectInputOptions(local, NULL, NULL);
     xf86OptionListReport(local->options);
 
     /* Joystick device is mandatory */
     priv->device = xf86SetStrOption(dev->commonOptions, "Device", NULL);
+    if (!priv->device)
+        priv->device = xf86SetStrOption(dev->commonOptions, "Path", NULL);
 
     if (!priv->device) {
         xf86Msg (X_ERROR, "%s: No Device specified.\n", local->name);
@@ -539,6 +635,9 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
         xfree(s);
     }
 
+    priv->mouse_enabled = xf86SetBoolOption(dev->commonOptions, "StartMouseEnabled", TRUE);
+    priv->keys_enabled = xf86SetBoolOption(dev->commonOptions, "StartKeysEnabled", TRUE);
+
     /* Process button mapping options */
     for (i=0; i<MAXBUTTONS; i++) {
         char p[64];
@@ -569,6 +668,11 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     /* return the LocalDevice */
     local->flags |= XI86_CONFIGURED;
 
+    priv->keyboard_device = jstkKeyboardPreInit(&JSTK_KEYBOARD, dev, flags);
+    if (priv->keyboard_device) {
+        priv->keyboard_device->private = priv;
+    }
+
     return (local);
 
 SetupProc_fail:
@@ -576,7 +680,8 @@ SetupProc_fail:
         xfree(priv);
     if (local)
         local->private = NULL;
-    return (local);
+    return NULL;
+/*    return (local); */ /* Makes X segfault on error */
 }
 
 
@@ -598,7 +703,11 @@ jstkCoreUnInit(InputDriverPtr    drv,
 {
     JoystickDevPtr device = (JoystickDevPtr) local->private;
 
-    jstkDeviceControlProc(local->dev, DEVICE_OFF);
+    if (device->keyboard_device != NULL)
+    {
+        xf86DisableDevice(device->keyboard_device->dev, TRUE);
+        device->keyboard_device = NULL;
+    }
 
     xfree (device);
     local->private = NULL;
@@ -636,6 +745,7 @@ jstkDriverPlug(pointer  module,
                int      *errmin)
 {
     xf86AddInputDriver(&JOYSTICK, module, 0);
+    xf86AddInputDriver(&JSTK_KEYBOARD, module, 0);
     return module;
 }
 
